@@ -95,7 +95,7 @@ waldtest.default <- function(object, ..., vcov = NULL, test = c("F", "Chisq"), n
 
     ## compute Chisq statistic
     stat <- t(coef(fm1)[ovar]) %*% solve(vc[ovar,ovar]) %*% coef(fm1)[ovar]
-    return(c(q, stat))
+    return(c(-q, stat))
   }
 
   ## recursively fit all objects (if necessary)
@@ -105,6 +105,12 @@ waldtest.default <- function(object, ..., vcov = NULL, test = c("F", "Chisq"), n
     objects <- c(objects, . ~ 1)
     nmodels <- 2
   }
+  
+  # remember which models are already fitted and which are described
+  # by an update mechanism
+  no.update <- sapply(objects, function(obj) inherits(obj, cls))
+  
+  ## updating
   for(i in 2:nmodels) objects[[i]] <- modelUpdate(objects[[i-1]], objects[[i]])
 
   ## check responses
@@ -118,7 +124,20 @@ waldtest.default <- function(object, ..., vcov = NULL, test = c("F", "Chisq"), n
 
   ## check sample sizes
   ns <- sapply(objects, nobs)
-  if(any(ns != ns[1])) stop("models were not all fitted to the same size of dataset")
+  if(any(ns != ns[1])) {
+    for(i in 2:nmodels) {
+      if(ns[1] != ns[i]) {
+        if(no.update[i]) stop("models were not all fitted to the same size of dataset")
+	  else {
+	    commonobs <- row.names(model.frame(objects[[i]])) %in% row.names(model.frame(objects[[i-1]]))
+	    objects[[i]] <- eval(substitute(update(objects[[i]], subset = commonobs),
+	      list(commonobs = commonobs)))
+	    if(nobs(objects[[i]]) != ns[1]) stop("models could not be fitted to the same size of dataset")
+	  }
+      }
+    }
+  }
+
   ## check vcov.
   if(nmodels > 2 && !is.null(vcov.) && !is.function(vcov.))
     stop("to compare more than 2 models `vcov.' needs to be a function")
@@ -134,7 +153,7 @@ waldtest.default <- function(object, ..., vcov = NULL, test = c("F", "Chisq"), n
     rval[,4] <- pchisq(rval[,3], round(abs(rval[,2])), lower.tail = FALSE)
   } else {
     df <- rval[,1]
-    for(i in 2:nmodels) if(rval[i,2] > 0) df[i] <- rval[i-1,1]
+    for(i in 2:nmodels) if(rval[i,2] < 0) df[i] <- rval[i-1,1]
     rval[,3] <- rval[,3]/abs(rval[,2])
     rval[,4] <- pf(rval[,3], abs(rval[,2]), df, lower.tail = FALSE)
   }
